#ifndef CCONTROLLER_H
#define CCONTROLLER_H

//------------------------------------------------------------------------
//
//	Name: CController.h
//
//  Code by: George Taylor 2013
//
//  Based on a framework provided by: Mat Buckland 2002
//
//  Desc: Controller class for the 'Neuarl Gnat' example 
//
//------------------------------------------------------------------------
#include <vector>
#include <sstream>
#include <string>
#include <fstream>
#include <windows.h>

#include "CNeuralGnats.h"
#include "CGenAlg.h"
#include "utils.h"
#include "C2DMatrix.h"
#include "SVector2D.h"
#include "CParams.h"

using namespace std;

#define NUM_QUADS 8

class CController
{

private:

	//storage for the population of genomes
	vector<SGenome>	     m_vecThePopulation;

	//and the NeuralGnats
    vector<CNeuralGnat> m_vecGnats;

	//and the fungi
	vector<SVector2D>	   m_vecFungi;
	vector<SVector2D>	   m_vecStartPositions;
	vector<int>	           m_vecStartPositionIdexes;

	//pointer to the GA
	CGenAlg*		         m_pGA;

	int					         m_NumGnats;
	int					         m_NumFungi;
	int					         m_NumWeightsInNN;
	int m_Num_Saves;
	int m_Num_Loads;

	//vertex buffer for the gnat's shape's vertices
	vector<SPoint>		   m_GnatVB;

	//vertex buffer for the fungus shape's vertices
	vector<SPoint>		   m_FungusVB;

	//stores the average fitness per generation for use 
	vector<double>		   m_vecRollingAvFitness;
	vector<double>		   m_vecRollingBestFitness;
	RECT                   rFungusQuads[NUM_QUADS];


	double m_dBestFitnessRollingAvg;
    double m_dAvgFitnessRollingAvg;
    double m_Composite;
    double m_BestComposite;
    double dBestFitness;
    double dBestWorstFitnessRatio;
    double m_dBestAvgRollingRatio;
    double m_ExpectedEliteWinsRatio;
    double m_dScalefactor;
	double m_dAvgGeneticDifference;

	//pens we use for the stats
	HPEN				m_RedPen;
	HPEN				m_BluePen;
	HPEN				m_GreenPen;
	HPEN                m_YellowPen;
	HPEN				m_OldPen;
	
	//handle to the application window
	HWND				m_hwndMain;
	
	//cycles per generation
	int					m_iTicks;
	int					m_curQuad;

	//generation counter
	int					m_iGenerations;
	int					m_iPrevGenerations;
    int                 m_EliteWins;
    //window dimensions
    int         cxClient, cyClient;
	//toggles the speed at which the simulation runs
	bool		m_bFastRender;
	bool        bUpdated;
	bool        bSaveWeights;
	bool        bSaveAllWeights;
	bool        bLoadWeights;
	bool        bLoadAllWeights;
	bool        bShowVectors;

  void   DrawGnat(HDC surface, int gnatIndex);
  void   DrawFungi(HDC surface);
  void   PlotStats(HDC surface);
  void   SaveBestWeights(int index);
  void   LoadBestWeights(int index);
  void   GetRandomFungusPosition(int *pX, int *pY, int quad);
  void   GetRandomGnatPosition(int *pX, int *pY, int quad, int iGnat);
  bool   FungusIsCloseToGnat(SVector2D &vFungusPos);
  bool   FungusIsOutsideRadius(SVector2D &vFungusPos);
  bool   GnatIsCloseToOthers(SVector2D &vGnatPos, int curGnat);
  void   RandomizeGnatStartPositions(void);
  void   RandomizeStartPositionIndexes(void);

public:

	CController(HWND hwndMain);

	~CController();

	bool		NeedUpdate(void);

	void		Render(HDC surface);

	void		WorldTransform(vector<SPoint> &VBuffer,
                         SVector2D      vPos);
	
	bool		Update();


	// accessor methods
	bool		FastRender()const	  {return m_bFastRender;}
	void		FastRender(bool arg){m_bFastRender = arg;}
	void		FastRenderToggle()  {m_bFastRender = !m_bFastRender;}
	void		ShowVectorsToggle()  {bShowVectors = !bShowVectors;}
	void        SaveWeights(void){bSaveWeights = true;}
	void        LoadWeights(void){bLoadAllWeights = true;}
    void        SaveAllWeights(void);
    void        LoadAllWeights(void);

};

#endif
	
