#include "stdafx.h"
#include "CNeuralGnats.h"
#include <stddef.h>
#include "Debug.h"

#define FROZEN_PERMITTER_WALL 1
#define FROZEN_FORBIDDEN_ZONE 2
#define FROZEN_SIGNAL -1 // -1 is a much better signal than 0)


//***************************************************************
// Define ONLY ONE of these three compiler switches,
// leaving the remainder commented out!
// You must remember to set up the params.ini file appropriately in each cases.
// (See notes above the funtion defined below in each case)
// #define SINGLE_INPUT
#define TWO_INPUTS
// #define FIVE_INPUTS 
//***************************************************************

// Define this if you hope to see the gnats spontaneously develop a protocol
// to take turns when there is contention for food.
// Otherwise, comment it out, and the gnats will be told
// who is closer to the food in such a situation.
#define TAKE_TURNS


static SVector2D       m_vCentre;
static LINE            m_Lines[MAX_LINES];
static LINE            m_LineAcross;
static bool            *m_RejectedFungi;
static double          m_dMaxDimension;
static double          m_dOneThirdMaxDimension;
static double          m_dTwoThirdsMaxDimension;
static double          m_dLongestPossibleSeparation;
static bool            bInitializedStaticVars = false;


//-----------------------------------constructor-------------------------
//
//-----------------------------------------------------------------------
CNeuralGnat::CNeuralGnat(SVector2D vStartPos, int arrayIndex):
                             m_vWorldPosition(vStartPos),
                             m_dFitness(0),
							 m_dScale(CParams::iGnatScale)
			 
{
    if(!bInitializedStaticVars)
	{
		m_dMaxDimension = CParams::WindowWidth - 1; // Assumes a square window
		m_vCentre.x = m_dMaxDimension / 2.0;
		m_vCentre.y = m_dMaxDimension / 2.0;
		m_dOneThirdMaxDimension = m_dMaxDimension / 3.0;
		m_dTwoThirdsMaxDimension = m_dOneThirdMaxDimension * 2.0;

		int i = 0;
/*
		// Permitter...
		// From top on going clockwise
		m_Lines[i].x1 = 0;
		m_Lines[i].x2 = m_dMaxDimension;
		m_Lines[i].y1 = 0;
		m_Lines[i++].y2 = 0;

		m_Lines[i].x1 = m_dMaxDimension;
		m_Lines[i].x2 = m_dMaxDimension;
		m_Lines[i].y1 = 0;
		m_Lines[i++].y2 = m_dMaxDimension;

		m_Lines[i].x1 = 0;
		m_Lines[i].x2 = m_dMaxDimension;
		m_Lines[i].y1 = m_dMaxDimension;
		m_Lines[i++].y2 = m_dMaxDimension;

		m_Lines[i].x1 = 0;
		m_Lines[i].x2 = 0;
		m_Lines[i].y1 = 0;
		m_Lines[i++].y2 = m_dMaxDimension;
*/
		// Forbidden Zone runs between 170.33333... and 340.66666..., inclusivly.
		// From top on going clockwise
		m_Lines[i].x1 = m_dOneThirdMaxDimension;
		m_Lines[i].x2 = m_dTwoThirdsMaxDimension;
		m_Lines[i].y1 = m_dOneThirdMaxDimension;
		m_Lines[i++].y2 = m_dOneThirdMaxDimension;

		m_Lines[i].x1 = m_dTwoThirdsMaxDimension;
		m_Lines[i].x2 = m_dTwoThirdsMaxDimension;
		m_Lines[i].y1 = m_dOneThirdMaxDimension;
		m_Lines[i++].y2 = m_dTwoThirdsMaxDimension;

		m_Lines[i].x1 = m_dOneThirdMaxDimension;
		m_Lines[i].x2 = m_dTwoThirdsMaxDimension;
		m_Lines[i].y1 = m_dTwoThirdsMaxDimension;
		m_Lines[i++].y2 = m_dTwoThirdsMaxDimension;

		m_Lines[i].x1 = m_dOneThirdMaxDimension;
		m_Lines[i].x2 = m_dOneThirdMaxDimension;
		m_Lines[i].y1 = m_dOneThirdMaxDimension;
		m_Lines[i].y2 = m_dTwoThirdsMaxDimension;

	    m_RejectedFungi = new bool [CParams::iNumFungi];
		ZeroMemory(&m_LineAcross, sizeof(LINE));
        // Calculate the longest possible distance between gnats & fungus or between gnats and another closest gnat.
		// It will be a line drawn from a corner, passing just outside a corner of the inner square, to the mid point on the far wall.
		// In other words, it will be the hypotenuse of a rectangle 512 x 256 pixels...
		m_dLongestPossibleSeparation = sqrt((double)((512 * 512) + (256 * 256)));
		bInitializedStaticVars = true;
	}

    m_iMyIndex = arrayIndex;

	// Reset the gnats positions
	Reset(vStartPos);
}


bool CNeuralGnat::PointOnLine(LINE const &line, SVector2D const &vPoint)
{
#define FINE_TOLERANCE 0.00000001
#define COURSE_TOLERANCE 0.000001

 double startX, startY;
 double endX, endY;

 if(line.x1 < line.x2)
 {
    startX = line.x1;
	endX = line.x2;
 }else
 {
    startX = line.x2;
	endX = line.x1;
 }

 if(line.y1 < line.y2)
 {
    startY = line.y1;
	endY = line.y2;
 }else
 {
    startY = line.y2;
	endY = line.y1;
 }

 // Is the point out of bounds?

 if((vPoint.x < (startX - FINE_TOLERANCE)) || (vPoint.x > (endX + FINE_TOLERANCE)))
 {
	 return false;
 }

 if((vPoint.y < (startY - FINE_TOLERANCE)) || (vPoint.y > (endY + FINE_TOLERANCE)))
 {
	 return false;
 }

 // We could have an exceedingly small or zero delta when the rotation is due North, East, South, or West.
 // If it results in len < dTolerance, there will be no problem - as long as the point passes bounds checks.
 // However, if the delta is just over or equal to dTolerance, we may get an error.
 // The delta test is the weakest link due to this, because when it comes to determining the ratio derived
 // from one infinitesimly small number divided by another infinitesimly small number, who knows what it will be.
 // I think here the tolerance is more critical than in the code above, where we are only doing simple comparisons.
 // At this moment I am using a tolerance of 0.00000001, and it seems to work great for everything but these situations.

 double xLen = endX - startX;
 double yLen = endY - startY;

 // Is the line x length or y length essentially zero?
 // Since we have done bounds checking, we have a short cut out,
 // thus avoiding potential for divide by zero...
 if((xLen < COURSE_TOLERANCE) || (yLen < COURSE_TOLERANCE))
 {
     return true;
 }

 double xDelta = (vPoint.x - startX) / xLen;
 double yDelta = (vPoint.y - startY) / yLen;

 if(line.x1 > line.x2)
 {
    xDelta = 1.0 - xDelta;
 }

 if(line.y1 > line.y2)
 {
    yDelta = 1.0 - yDelta;
 }

 // x & y deltas should be essentially equal...
 if((xDelta < (yDelta - COURSE_TOLERANCE)) || (xDelta > (yDelta + COURSE_TOLERANCE)))
 {
    return false;
 }

 return true;
}


// See http://en.wikipedia.org/wiki/Line-line_intersection
// The intersection of two lines L1 and L2 in 2 dimensional space.
// With line L1 being defined by two points (x1, y1) and (x2, y2),
// and line L2 being defined by two points  (x3, y3) and (x4, y4),
// the intersection P of L1 and L2 can be determined by...
/*

(Px, Py) = { ((x1y2 - y1x2)(x3 - x4) - (x1 - x2)(x3y4 - y3x4)) / ((x1 - x2)(y3 - y4) - (y1 - y2)(x3 - x4)),
             ((x1y2 - y1x2)(y3 - y4) - (y1 - y2)(x3y4 - y3x4)) / ((x1 - x2)(y3 - y4) - (y1 - y2)(x3 - x4)) }

*Note: When the two lines are parallel or coincident the denominator term is zero:
((x1 - x2)(y3 - y4) - (y1 - y2)(x3 - x4)) = 0

Note that the intersection point is for the infinitely long lines defined by the points,
rather than the line segments between the points, and can produce an intersection point beyond the lengths of the line segments.
*/


bool CNeuralGnat::FindIntersectionPoint(LINE const &line, SVector2D const &vPosition, SVector2D const &vProjection, SVector2D *pvIntersectionPoint)
{
 SVector2D vStart(0, 0);
 SVector2D vEnd(0, 0);
 SVector2D vMagA(0, 0);
 SVector2D vMagB(0, 0);
 double termA, termB, divisor;

 vStart.x = line.x1;
 vStart.y = line.y1;
 vEnd.x = line.x2;
 vEnd.y = line.y2;


 vMagA = vProjection - vPosition;
 vMagB = vEnd - vStart;
 divisor = (vMagA.x * vMagB.y) - (vMagA.y * vMagB.x);
 if(divisor == 0)return false;
 // do (x1y2 - y1x2)
 termA = (vPosition.x * vProjection.y) - (vPosition.y * vProjection.x);
 // do (x3y4 - y3x4)
 termB = (vStart.x * vEnd.y) - (vStart.y * vEnd.x);

 // do ((x1y2 - y1x2)(x3 - x4) - (x1 - x2)(x3y4 - y3x4)) / divisor
 // = termA(x3 - x4) - (x1 - x2)(x3y4 - y3x4))
 // = (termA * vMagB.x) - (vMagA.x * termB)
 pvIntersectionPoint->x = fabs(((termA * vMagB.x) - (vMagA.x * termB)) / divisor);

 // do ((x1y2 - y1x2)(y3 - y4) - (y1 - y2)(x3y4 - y3x4)) / divisor
 // = termA(y3 - y4) - (y1 - y2)(x3y4 - y3x4))
 // = (termA * vMagB.y) - (vMagA.y * termB)
 pvIntersectionPoint->y = fabs(((termA * vMagB.y) - (vMagA.y  * termB)) / divisor);

 return true;
}

bool CNeuralGnat::PathToObjectIntersectsWall(SVector2D &vObjectWorldPos, double dDistanceObjectToCentre)
{
 SVector2D vIntersectionPoint(0, 0);
 LINE lProjection;
 double startX, startY;
 double endX, endY;

 // If the combined distances from centre exceeds an emphirical value
 // derived from observation, the objects are too far from the centre
 // square to be concerned that the path between them would intersect with it...
 if((dDistanceObjectToCentre + m_dDistanceGnatToCentre) > 381.0)
 {
    // A majority of the calls to this function exit here... 
    return false;
 }

 // Project a line from current position to the object.
 // The object's position is the projected point
 if(m_vWorldPosition.x < vObjectWorldPos.x)
 {
    startX = m_vWorldPosition.x;
	endX = vObjectWorldPos.x;
 }else
 {
    startX = vObjectWorldPos.x;
	endX = m_vWorldPosition.x;
 }

 if(m_vWorldPosition.y < vObjectWorldPos.y)
 {
    startY = m_vWorldPosition.y;
	endY = vObjectWorldPos.y;
 }else
 {
    startY = vObjectWorldPos.y;
	endY = m_vWorldPosition.y;
 }

 // If the projected line does not pass thru the Forbidden Zone
 // there is no point in checking for collision with Forbidden Zone walls...
 if( ((startX < m_dOneThirdMaxDimension) && (endX < m_dOneThirdMaxDimension))  || 
	 ((startX > m_dTwoThirdsMaxDimension) && (endX > m_dTwoThirdsMaxDimension)) )
 {
     return false;
 }

 if( ((startY < m_dOneThirdMaxDimension) && (endY < m_dOneThirdMaxDimension))  || 
	 ((startY > m_dTwoThirdsMaxDimension) && (endY > m_dTwoThirdsMaxDimension)) )
 {
     return false;
 }

 // 98.34 percent of the calls that survived the distance test
 // are filtered out by the above tests... 
 // Only 1.66% of calls pass the test just above

 lProjection.x1 = m_vWorldPosition.x;
 lProjection.x2 = vObjectWorldPos.x;
 lProjection.y1 = m_vWorldPosition.y;
 lProjection.y2 = vObjectWorldPos.y;

 for(int i = 0; i < MAX_LINES; i++)
 {
     if(FindIntersectionPoint(m_Lines[i], m_vWorldPosition, vObjectWorldPos, &vIntersectionPoint))
	 {
        // Is the point within the bounderies?
	    if(!PointOnLine(m_Lines[i], vIntersectionPoint))
	    {
           continue;
	    }

        if(!PointOnLine(lProjection, vIntersectionPoint))
	    {
           continue;
	    }

        // He will collide with the wall before he reaches the object.
        // 54% percent of paths that reach this test are found to cross a line
		// m_LineAcross = lProjection;
		return true;
	 }
 }
 // 46% percent of paths that reached this test are found not to cross a line
 return false;
}

// To display a line connecting the objects. Not currently used
void CNeuralGnat::GetLineAcross(int *pX1, int *pY1, int *pX2, int *pY2)
{
 *pX1 = (int)floor(m_LineAcross.x1 + 0.5);
 *pY1 = (int)floor(m_LineAcross.y1 + 0.5);
 *pX2 = (int)floor(m_LineAcross.x2 + 0.5);
 *pY2 = (int)floor(m_LineAcross.y2 + 0.5);
};

// To display the vectors supplied to the gnat press 'V'
void CNeuralGnat::GetDisplayedPoints(int *pX1, int *pY1, int *pX2, int *pY2)
{
 if(m_vDisplayedGnatPos.x >= 0)
 {
    *pX1 = (int)floor(m_vDisplayedGnatPos.x + 0.5);
    *pY1 = (int)floor(m_vDisplayedGnatPos.y + 0.5);
 }else
 {
    *pX1 = -1;
    *pY1 = -1;
 }
 if(m_vDisplayedFungusPos.x >= 0)
 {
    *pX2 = (int)floor(m_vDisplayedFungusPos.x + 0.5);
    *pY2 = (int)floor(m_vDisplayedFungusPos.y + 0.5);
 }else
 {
    *pX2 = -1;
    *pY2 = -1;
 }
}

//-------------------------------------------Reset()--------------------
//
//	Resets the gnats position, fitness and rotation
//
//----------------------------------------------------------------------
void CNeuralGnat::Reset(SVector2D vStartPos)
{
	// Reset the gnats positions
    m_vWorldPosition = vStartPos;
	m_vTranslatedPosition = vStartPos;
    Vec2DToTranslated(m_vTranslatedPosition, m_vCentre);
    m_dDistanceGnatToCentre = Vec2DLength(m_vTranslatedPosition);
    // Get the angle from centre to start position...
	m_dRotation = atan2(m_vTranslatedPosition.y, m_vTranslatedPosition.x);
    // Rotate 90 degrees to right
	m_dRotation += CParams::dHalfPi;
	m_dPrevRotation = m_dRotation;

	// Maintain m_dRotation between +/- Pi
	if(m_dRotation > CParams::dPi)
	{
       m_dRotation -= CParams::dTwoPi;
	}
    m_vDisplayedGnatPos.x = -1;
    m_vDisplayedGnatPos.y = -1;
	m_vDisplayedFungusPos.x = -1;
	m_vDisplayedFungusPos.y = -1;
	m_dFitness = 0;
	m_iFrozen = 0;
    m_iClosestGnat = -1;
	m_iClosestFungus = -1;

// In the case of SINGLE_INPUT or not TAKE_TURNS the organisms do not exhibit handedness...
#ifdef SINGLE_INPUT 

	m_iHandedness = LEFTHANDED; // To inform the displaying of the gnats only
	return;
}

#else

	#ifdef TAKE_TURNS

		// At the start of each round, half will be left handed
		// and the other half right handed. Of course their
		// "handedness" will quickly change as they encounter others.
		if((m_iMyIndex % 2) == 0)
		{
			m_iHandedness = LEFTHANDED;
		}else
		{
			m_iHandedness = RIGHTHANDED;
		}

	#else

		m_iHandedness = LEFTHANDED; // To inform the displaying of the gnats only

	#endif

	return;
}
#endif

//---------------------WorldTransform--------------------------------
//
//	sets up a translation matrix for the gnat according to its
//  scale, rotation and position. Returns the transformed vertices.
//-------------------------------------------------------------------
void CNeuralGnat::WorldTransform(vector<SPoint> &gnat, double dScale)
{
 double dRotation;

 // rotate: matrix manipulation seems to require an offset to the angle
 dRotation = m_dPrevRotation - CParams::dHalfPi;
 // Maintain m_dRotation between +/- Pi
 if(dRotation < -CParams::dPi)
 {
    dRotation += CParams::dTwoPi;
 }

 // create the world transformation matrix
 C2DMatrix matTransform;
	
 // scale
 matTransform.Scale(dScale, dScale);
	
 // rotate: matrix manipulation seems to require an offset to the angle
 matTransform.Rotate(dRotation);
	
 // and translate
 matTransform.Translate(m_vWorldPosition.x, m_vWorldPosition.y);
	
 // now transform the vertices
 matTransform.TransformSPoints(gnat);
}


// Offset angleA by adding the normalized magnitude of some angle to it and return the result
double CNeuralGnat::ReverseMagnitudeAngularDifferences(double dAngleA, double dMagnitudeAngularDifferences)
{
 double dAngleB, dRadians;

 dRadians = dMagnitudeAngularDifferences * CParams::dPi;
 // We clamp for aethstetic reasons, since a fly can't turn 180 degrees instantly
 // If he is frozen however, we allow him to rotate as much as he can.
 if(m_iFrozen == 0)
 {
	 Clamp(dRadians, -CParams::dMaxTurnRate, CParams::dMaxTurnRate); // About 57 degrees for CParams::dMaxTurnRate = 1.0
 }

 dAngleB = dAngleA + dRadians;
 if(dAngleB > CParams::dPi)
 {
    dAngleB -= CParams::dTwoPi;
 }else if(dAngleB <= -CParams::dPi)
 { 
    dAngleB += CParams::dTwoPi;
 }
 return dAngleB;
}

// If dAngleB is clockwise rotated from dAngleA, return normalized positive magnitude of that offset.
// If dAngleB is counterclockwise rotated from dAngleA, return normalized negative magnitude of that offset.
// Current usage:
// inputs.push_back(MagnitudeAngularDifferences(m_dRotation, m_ClosestFungusAngle));
double CNeuralGnat::MagnitudeAngularDifferences(double dAngleA, double dAngleB)
{
 double dDistance;

 if(((dAngleA >= 0) && (dAngleB >= 0)) || ((dAngleA < 0) && (dAngleB < 0)))
 {
	 if(dAngleA > dAngleB)
	 {
        dDistance = dAngleA - dAngleB;
		dDistance *= -1;
	 }else
	 {
        dDistance = dAngleB - dAngleA;
	 }
 }else
 {
     double dClockwiseDistance, dCounterClockwiseDistance;
	 if(dAngleA >= 0)
	 {
        // dAngleB is negative
        dClockwiseDistance = CParams::dPi - dAngleA;
		dClockwiseDistance += (CParams::dPi + dAngleB);
		dCounterClockwiseDistance = dAngleA;
		dCounterClockwiseDistance += -dAngleB;
	 }else
	 {
        // dAngleB is positive
        dClockwiseDistance = -dAngleA;
		dClockwiseDistance += dAngleB;
		dCounterClockwiseDistance = CParams::dPi + dAngleA;
		dCounterClockwiseDistance += (CParams::dPi - dAngleB);
	 } 
	 if(dClockwiseDistance <= dCounterClockwiseDistance)
	 {
	    dDistance = dClockwiseDistance;
	 }else
	 {
	    dDistance = dCounterClockwiseDistance;
		dDistance *= -1;
	 }
 }

 // Normalize...
 return dDistance / CParams::dPi;
}



#ifdef SINGLE_INPUT

// This is the simplest organism.
// It receives the direction to the nearest food as an offset to current heading on its only input.
// You must set the params.ini file as follows...
// iNumInputs 1
// iNumHidden 1  
// iNeuronsPerHiddenLayer 2  <-- You can experiment with this value
// iNumOutputs 2
// These organisms should easily reach 325 point group score within 1000 generations,
// but it could take as much as a couple of hundred generations
// for them to learn to navigate towards food sometime.
bool CNeuralGnat::Update(vector<SVector2D> &fungi, vector<CNeuralGnat> &gnats, bool bShow)
{
 vector<double> inputs; // <-- This will store all the inputs for the Neural Net

 if(bShow)m_dPrevRotation = m_dRotation; // m_dPrevRotation used for displaying only

 if(m_iFrozen > 0)
 {
    // On the last move, he tried to take a step but it would have
	// landed him on or beyond a wall, where he is forbidden to go.
    // His position is "frozen" until he takes a step away from the wall,
	// though he can still rotate on the spot.

	// Ring all his alarm bell.
	inputs.push_back(FROZEN_SIGNAL);

    // Update the brain and get feedback
    vector<double> output = m_ItsBrain.Update(inputs);

    // Make sure there were no errors in calculating the output
    if((int)output.size() < CParams::iNumOutputs) 
    {
        return false;
    }

    UpdatePosition(output[0], output[1]);

	// Is he still stuck?
    if(m_iFrozen > 0)
	{
       if(bShow)
	   {
	      // used for displaying only
	      m_vDisplayedFungusPos.x = -1;
	      m_vDisplayedFungusPos.y = -1;
          m_vDisplayedGnatPos.x = -1;
          m_vDisplayedGnatPos.y = -1;
	   }
	   return true;
	}

	// If he managed to get unstuck, he gets another step as a reward for his efforts
	inputs.clear();
 }

 // Gather input data for the neural net...

 double dDistanceToCentre; // <-- Optimization for function PathToObjectIntersectsWall()
 GetClosestFungus(fungi, &dDistanceToCentre); // Sets m_iClosestFungus, m_ClosestFungusAngle, m_dDistanceToFungus

 // Does a line drawn from the gnat to the closest fungus pass thru a corner of the inner square?
 if(PathToObjectIntersectsWall(fungi[m_iClosestFungus], dDistanceToCentre))
 {
    // A gnat doesn't doesn't have sufficient information to understand that it is impossible for him
    // to reach a fungus when there is an obstacle in his path, namely, a corner of the "forbidden zone" (inner square). 
    // He would just end up banging his head on the wall until circumstances change.
    GetFungusAwayFromWall(fungi); // Sets m_iClosestFungus, m_ClosestFungusAngle
 } 

 // Input direction to food as the normalized magnitude of the difference between
 // current heading, and the heading to the food.
 inputs.push_back(MagnitudeAngularDifferences(m_dRotation, m_ClosestFungusAngle));


 if(bShow)
 {
    // used for displaying only when user togles the 'V' key
    m_vDisplayedFungusPos = fungi[m_iClosestFungus];
 }


 // Update the brain and get feedback
 vector<double> output = m_ItsBrain.Update(inputs);

 // Make sure there were no errors in calculating the output
 if((int)output.size() < CParams::iNumOutputs) 
 {
     return false;
 }

 UpdatePosition(output[0], output[1]);

 return true;  
}
#endif



#ifdef TWO_INPUTS
// This organism receives the direction to the nearest food as an offset to current heading on one input
// and an alert to a conflict with a neighbour over food on the second input described as the "Identity Input".

// When a neighbour is encuntered, if the current organism happend to be the same handedness,
// its handedness is changed to the opposite.

// If TAKE_TURNS is defined, the organism's current handedness is input on the "Identity Input",
// and it is up to the organisms what to do with this information.
// Perhaps they will develop turn-taking. Perhaps not.

// In a quick test done as I was organizing the code just now,
// these organisms developed turn taking within the first couple of hundred generations.
// By 1000 generations they were actually outscoring the colony that was told who
// was closer to the food - reaching a score of 480 points.
// By 2000 generations their score reached 500, but in the long run the others have the potential
// of reaching maybe 630 points, whereas these will top out 100 points lower.

// If TAKE_TURNS is NOT defined, then the organism is told exactly who is closest to the food
// by placing a -1 (as defeined by RIGHTHANDED) on his input if he is closer,
// or a 1 (as defined by LEFTHANDED) on his identity input if the neighbour is closer.
// This makes it a no brainer for the gnats to know whether to go for the food or look elsewhere,
// and there is no need of it developing some social protocol.
// These organisms should easily reach a 460 point group score within 1000 generations.


// You must set the params.ini file as follows...
// iNumInputs 2
// iNumHidden 1  
// iNeuronsPerHiddenLayer 2  <-- You can experiment with this value
// iNumOutputs 2

// Some comments have been omitted to reduce space where the code is identical to the code above.
bool CNeuralGnat::Update(vector<SVector2D> &fungi, vector<CNeuralGnat> &gnats, bool bShow)
{
 vector<double> inputs; // <-- This will store all the inputs for the Neural Net

 if(bShow)m_dPrevRotation = m_dRotation; // m_dPrevRotation used for displaying only

 if(m_iFrozen > 0)
 {
	// Ring all his alarm bells.
	inputs.push_back(FROZEN_SIGNAL);
	inputs.push_back(FROZEN_SIGNAL);

    // Update the brain and get feedback
    vector<double> output = m_ItsBrain.Update(inputs);

    // Make sure there were no errors in calculating the output
    if((int)output.size() < CParams::iNumOutputs) 
    {
        return false;
    }

    UpdatePosition(output[0], output[1]);

	// Is he still stuck?
    if(m_iFrozen > 0)
	{
       if(bShow)
	   {
	      m_vDisplayedFungusPos.x = -1;
	      m_vDisplayedFungusPos.y = -1;
          m_vDisplayedGnatPos.x = -1;
          m_vDisplayedGnatPos.y = -1;
	   }
	   return true;
	}

	// If he managed to get unstuck, he gets another step as a reward for his efforts
	inputs.clear();
 }

 // Gather input data for the neural net...

 double dDistanceToCentre; // <-- Optimization for function PathToObjectIntersectsWall()
 GetClosestFungus(fungi, &dDistanceToCentre); // Sets m_iClosestFungus, m_ClosestFungusAngle, m_dDistanceToFungus

 // Does a line drawn from the gnat to the closest fungus pass thru a corner of the inner square?
 if(PathToObjectIntersectsWall(fungi[m_iClosestFungus], dDistanceToCentre))
 {
    GetFungusAwayFromWall(fungi); // Sets m_iClosestFungus, m_ClosestFungusAngle
 } 

 inputs.push_back(MagnitudeAngularDifferences(m_dRotation, m_ClosestFungusAngle));

 GetClosestGnat(gnats); // Sets m_iClosestGnat, m_ClosestGnatAngle, m_dDistanceToGnat
 if((m_iClosestFungus == gnats[m_iClosestGnat].m_iClosestFungus) ||
     HaveCollidingObjectives(gnats[m_iClosestGnat].m_vTranslatedFungusPosition) )
 {
     // The neighbour of this gnat is after the same piece of food he wants
#ifdef TAKE_TURNS
	 // Just signal to the gnat there is a confict by inputting its current handednesss.
	 // Whether they make use of this information is up to them.
     if(gnats[m_iClosestGnat].m_iHandedness == m_iHandedness)
     {
        m_iHandedness = -m_iHandedness;
     }

     inputs.push_back(m_iHandedness);
#else
	 // Just signal to the gnat there is a confict over the food
	 // with a -1 if he is closer to the food,
	 // or 1 if the competing neighbour is closer.
	 // This makes it a no brainder for the gnat to decide what to do.
	 if(m_dDistanceToFungus <= gnats[m_iClosestGnat].m_dDistanceToFungus)
	 {
        inputs.push_back(RIGHTHANDED);
	 }else inputs.push_back(LEFTHANDED);
#endif

     if(bShow)
	 {
	    // used for displaying only
        m_vDisplayedFungusPos = fungi[m_iClosestFungus];
        m_vDisplayedGnatPos = gnats[m_iClosestGnat].Position();
	 }
 }else
 {
    // Signal that there is no contention over food 
    inputs.push_back(AMBIVALENT);

     if(bShow)
	 {
        m_vDisplayedFungusPos = fungi[m_iClosestFungus];
        m_vDisplayedGnatPos.x = -1;
        m_vDisplayedGnatPos.y = -1;
	 }
 }

 // Update the brain and get feedback
 vector<double> output = m_ItsBrain.Update(inputs);

 // Make sure there were no errors in calculating the output
 if((int)output.size() < CParams::iNumOutputs) 
 {
     return false;
 }

 UpdatePosition(output[0], output[1]);

 return true;  
}

#endif



#ifdef FIVE_INPUTS
// This organism receives the distance and direction to the nearest food as an offset to current heading on two inputs,
// the distance and direction to the nearest neighbour as an offset to current heading on two other inputs,
// and an alert to a conflict with a neighbour over food on the fifth input described as the "Identity Input".

// When a neighbour is encuntered, if the current organism happend to be the same handedness,
// its handedness is changed to the opposite.

// If TAKE_TURNS is defined, the organism's current handedness is input on the "Identity Input",
// and it is up to the organisms what to do with this information.
// Perhaps they will develop turn-taking. Perhaps not.

// If TAKE_TURNS is NOT defined, then the organism is told exactly who is closest to the food
// by placing a -1 (as defeined by RIGHTHANDED) on his input if he is closer,
// or a 1 (as defined by LEFTHANDED) on his identity input if the neighbour is closer.
// This makes it a no brainer for the gnats to know whether to go for the food or look elsewhere,
// and there is no need of it developing some social protocol.

// You must set the params.ini file as follows...
// iNumInputs 5
// iNumHidden 1  
// iNeuronsPerHiddenLayer 4  <-- You can experiment with this value
// iNumOutputs 2
bool CNeuralGnat::Update(vector<SVector2D> &fungi, vector<CNeuralGnat> &gnats, bool bShow)
{
 vector<double> inputs; // <-- This will store all the inputs for the Neural Net

 if(bShow)m_dPrevRotation = m_dRotation; // m_dPrevRotation used for displaying only

 if(m_iFrozen > 0)
 {
    // On the last move, he tried to take a step but it would have
	// landed him on or beyond a wall, where he is forbidden to go.
    // His position is "frozen" until he takes a step away from the wall,
	// though he can still rotate on the spot.

	// Ring all his alarm bells.
	// None of these inputs ever see -1 except when he is frozen...
	inputs.push_back(FROZEN_SIGNAL);
	inputs.push_back(FROZEN_SIGNAL);
	inputs.push_back(FROZEN_SIGNAL);
	inputs.push_back(FROZEN_SIGNAL);
    inputs.push_back(AMBIVALENT);

    // Update the brain and get feedback
    vector<double> output = m_ItsBrain.Update(inputs);

    // Make sure there were no errors in calculating the output
    if((int)output.size() < CParams::iNumOutputs) 
    {
        return false;
    }

    UpdatePosition(output[0], output[1]);

	// Is he still stuck?
    if(m_iFrozen > 0)
	{
       if(bShow)
	   {
	      m_vDisplayedFungusPos.x = -1;
	      m_vDisplayedFungusPos.y = -1;
          m_vDisplayedGnatPos.x = -1;
          m_vDisplayedGnatPos.y = -1;
	      m_iHandedness = LEFTHANDED;
	   }
	   return true;
	}

	// If he managed to get unstuck, he gets another step as a reward for his efforts
	inputs.clear();
 }

 // Gather input data for the neural net...

 double dDistanceToCentre; // <-- Optimization for function PathToObjectIntersectsWall()
 GetClosestFungus(fungi, &dDistanceToCentre); // Sets m_iClosestFungus, m_ClosestFungusAngle, m_dDistanceToFungus

 // Does a line drawn from the gnat to the closest fungus pass thru a corner of the inner square?
 if(PathToObjectIntersectsWall(fungi[m_iClosestFungus], dDistanceToCentre))
 {
    // A gnat doesn't doesn't have sufficient information to understand that it is impossible for him
    // to reach a fungus when there is an obstacle in his path, namely, a corner of the "forbidden zone" (inner square). 
    // He would just end up banging his head on the wall until circumstances change.
    GetFungusAwayFromWall(fungi); // Sets m_iClosestFungus, m_ClosestFungusAngle
 } 

 GetClosestGnat(gnats); // Sets m_iClosestGnat, m_ClosestGnatAngle, m_dDistanceToGnat

 inputs.push_back(MagnitudeAngularDifferences(m_dRotation, m_ClosestFungusAngle));
 inputs.push_back(m_dDistanceToFungus / m_dLongestPossibleSeparation);
 inputs.push_back(MagnitudeAngularDifferences(m_dRotation, m_ClosestGnatAngle));
 inputs.push_back(m_dDistanceToGnat / m_dLongestPossibleSeparation);

 if((m_iClosestFungus == gnats[m_iClosestGnat].m_iClosestFungus) ||
     HaveCollidingObjectives(gnats[m_iClosestGnat].m_vTranslatedFungusPosition) )
 {
     // The neighbour of this gnat is after the same piece of food he wants
#ifdef TAKE_TURNS
	 // Just signal to the gnat there is a confict by inputting its current handednesss.
	 // Whether they make use of this information is up to them.
     if(gnats[m_iClosestGnat].m_iHandedness == m_iHandedness)
     {
        m_iHandedness = -m_iHandedness;
     }

     inputs.push_back(m_iHandedness);
#else
	 // Just signal to the gnat there is a confict over the food
	 // with a -1 if he is closer to the food,
	 // or 1 if the competing neighbour is closer.
	 // This makes it a no brainder for the gnat to decide what to do.
	 if(m_dDistanceToFungus <= gnats[m_iClosestGnat].m_dDistanceToFungus)
	 {
        inputs.push_back(RIGHTHANDED);
	 }else inputs.push_back(LEFTHANDED);
#endif


     if(bShow)
	 {
	    // used for displaying only
        m_vDisplayedFungusPos = fungi[m_iClosestFungus];
        m_vDisplayedGnatPos = gnats[m_iClosestGnat].Position();
	 }

 }else
 {

    inputs.push_back(AMBIVALENT);

     if(bShow)
	 {
	    // used for displaying only
        m_vDisplayedFungusPos = fungi[m_iClosestFungus];
        m_vDisplayedGnatPos.x = -1;
        m_vDisplayedGnatPos.y = -1;
	 }
 }

 // Update the brain and get feedback
 vector<double> output = m_ItsBrain.Update(inputs);

 // Make sure there were no errors in calculating the output
 if((int)output.size() < CParams::iNumOutputs) 
 {
     return false;
 }

 UpdatePosition(output[0], output[1]);

 return true;  
}

#endif



void CNeuralGnat::UpdatePosition(double X, double Y)
{
 double dSpeed = sqrt((1.0 - X) + (1.0 - Y));
 m_dRotation = ReverseMagnitudeAngularDifferences(m_dRotation, (X - Y));
 double newXPos = m_vWorldPosition.x + (cos(m_dRotation) * dSpeed);
 double newYPos = m_vWorldPosition.y + (sin(m_dRotation) * dSpeed);

 if(!m_iFrozen)
 {
    CheckBounds(newXPos, newYPos);
 }else
 {
    CheckFrozenBounds(newXPos, newYPos);
 }

 if(!m_iFrozen)
 {
	 m_vWorldPosition.x = newXPos;
	 m_vWorldPosition.y = newYPos;
     m_vTranslatedPosition = m_vWorldPosition;
	 Vec2DToTranslated(m_vTranslatedPosition, m_vCentre);
     m_dDistanceGnatToCentre = Vec2DLength(m_vTranslatedPosition);
 }
}


// Called only when we are frozen
void CNeuralGnat::CheckFrozenBounds(double X, double Y)
{
 // Did we get unstuck yet? 
 if(m_iFrozen == FROZEN_PERMITTER_WALL)
 {
	if( (X < m_dMaxDimension)  &&
		(X > 0)                &&
		(Y < m_dMaxDimension)  &&
		(Y > 0)                )
	{
 	     m_iFrozen = 0;
	}
 }else // FROZEN_FORBIDDEN_ZONE
 {
	if( !((X >= m_dOneThirdMaxDimension)  &&
	 	 (X <= m_dTwoThirdsMaxDimension)) ||
		!((Y >= m_dOneThirdMaxDimension)  &&
		 (Y <= m_dTwoThirdsMaxDimension))  )
    {
 	     m_iFrozen = 0;
	}
 }
}

void CNeuralGnat::CheckBounds(double X, double Y)
{
 if( (X >= m_dMaxDimension) ||
 	 (X <= 0)               ||
	 (Y >= m_dMaxDimension) ||
	 (Y <= 0)                )
 {
      m_iFrozen = FROZEN_PERMITTER_WALL;
 }else
 {
	 if( (X >= m_dOneThirdMaxDimension)  &&
	 	 (X <= m_dTwoThirdsMaxDimension) &&
		 (Y >= m_dOneThirdMaxDimension)  &&
		 (Y <= m_dTwoThirdsMaxDimension)  )
       {
          m_iFrozen = FROZEN_FORBIDDEN_ZONE;
	   }
 }
}



void CNeuralGnat::GetClosestGnat(vector<CNeuralGnat> &gnats)
{
 double	closest_so_far = 99999;
 int iClosest = 0;

 for(unsigned int i = 0; i < gnats.size(); i++)
 {
	 if(i == m_iMyIndex)continue;
	 // If gnat is frozen, ignore it
	 if(gnats[i].m_iFrozen > 0)continue;
	 double len_to_object = Vec2DLength(gnats[i].Position() - m_vWorldPosition);

 	 if(len_to_object < closest_so_far)
	 {
		closest_so_far = len_to_object;
		iClosest = i;
	}
 }
 m_dDistanceToGnat = closest_so_far;
 m_iClosestGnat = iClosest;
 SVector2D vNearestCompetitorVector(0, 0);
 vNearestCompetitorVector = gnats[m_iClosestGnat].TranslatedPosition() - m_vTranslatedPosition;
 m_ClosestGnatAngle = atan2(vNearestCompetitorVector.y, vNearestCompetitorVector.x);
}


void CNeuralGnat::GetClosestFungus(vector<SVector2D> &fungi, double *pDistanceFromCentre)
{
 double	closest_so_far = 99999;
 int iClosest = 0;

 // cycle through fungi to find closest
 for(unsigned int i = 0; i < fungi.size(); i++)
 {
	 double len_to_object = Vec2DLength(fungi[i] - m_vWorldPosition);

	 if(len_to_object < closest_so_far)
	 {
		closest_so_far = len_to_object;
		iClosest = i;
	 }
 }
 m_dDistanceToFungus = closest_so_far;
 m_iClosestFungus = iClosest;
 SVector2D	vClosestObject(0, 0);
 SVector2D	vTranslatedFungusPos = fungi[m_iClosestFungus];
 Vec2DToTranslated(vTranslatedFungusPos, m_vCentre);
 m_vTranslatedFungusPosition = vTranslatedFungusPos;
 vClosestObject	= vTranslatedFungusPos - m_vTranslatedPosition;
 m_ClosestFungusAngle = atan2(vClosestObject.y, vClosestObject.x);
 *pDistanceFromCentre = Vec2DLength(vTranslatedFungusPos);
}


// Sets m_iClosestFungus
void CNeuralGnat::FindAnotherCloseFungus(vector<SVector2D> &fungi)
{
 double	closest_so_far = 99999;
 int iClosest = 0;

 // Cycle through fungi to find next closest
 for(unsigned int i = 0; i < fungi.size(); i++)
 {
	 if(m_RejectedFungi[i])continue;
	 double len_to_object = Vec2DLength(fungi[i] - m_vWorldPosition);

	 if(len_to_object < closest_so_far)
	 {
		closest_so_far = len_to_object;
		iClosest = i;
	 }
 }
 m_dDistanceToFungus = closest_so_far;
 m_iClosestFungus = iClosest;
}


void CNeuralGnat::GetFungusAwayFromWall(vector<SVector2D> &fungi)
{
 SVector2D	vClosestObject(0, 0);
 SVector2D	vTranslatedFungusPos(0, 0);
 double dDistanceFromCentre;
 int countRejected = 0; // for safety

/*
 if((m_iClosestFungus < 0) || (m_iClosestFungus >= CParams::iNumFungi))
 {
	 DebugValue("Error - GetFungusAwayFromWall m_iClosestFungus", m_iClosestFungus);
	 m_iClosestFungus = 0;
 }
*/

 ZeroMemory(m_RejectedFungi, sizeof(bool) * CParams::iNumFungi);
 // m_iClosestFungus was rejected before we got here
 m_RejectedFungi[m_iClosestFungus] = true;
 ++countRejected;
 FindAnotherCloseFungus(fungi);
 vTranslatedFungusPos = fungi[m_iClosestFungus];
 Vec2DToTranslated(vTranslatedFungusPos, m_vCentre);
 dDistanceFromCentre = Vec2DLength(vTranslatedFungusPos);

 // m_iClosestFungus was set to the next closest in FindAnotherCloseFungus()
 while(PathToObjectIntersectsWall(fungi[m_iClosestFungus], dDistanceFromCentre))
 {
       m_RejectedFungi[m_iClosestFungus] = true;
       FindAnotherCloseFungus(fungi);
       vTranslatedFungusPos = fungi[m_iClosestFungus];
       Vec2DToTranslated(vTranslatedFungusPos, m_vCentre);
       dDistanceFromCentre = Vec2DLength(vTranslatedFungusPos);
	   if(++countRejected >= CParams::iNumFungi)
	   {
		  DebugMessage("GetFungusAwayFromWall can't find a fungus!");
          // Something's wrong. All gnats on one side or at one end? - I don't believe it...
          break; // for safety, or we will go into an endless loop and blow the stack up
	   }    
 } 

// vTranslatedFungusPos = fungi[m_iClosestFungus]; done above
// Vec2DToTranslated(vTranslatedFungusPos, m_vCentre);
 m_vTranslatedFungusPosition = vTranslatedFungusPos;
 vClosestObject	= vTranslatedFungusPos - m_vTranslatedPosition;
 m_ClosestFungusAngle = atan2(vClosestObject.y, vClosestObject.x);
}

// Using this gave them a 30 point boost in one test - (from 488.98242 to 521.29687 not averaged (516.954 smoothed))
bool CNeuralGnat::HaveCollidingObjectives(SVector2D &vNeighbourTranslatedFungusPos)
{
 SVector2D	vDistanceBetweenFungi(0, 0);
 double dSeparation;
 vDistanceBetweenFungi = m_vTranslatedFungusPosition - vNeighbourTranslatedFungusPos;
 dSeparation = Vec2DLength(vDistanceBetweenFungi);
 if(dSeparation <= 20.0)return true;
 return false;
}


//----------------------------- CheckForFungus -----------------------------
//
//  This function checks for collision with its closest fungus (calculated
//  earlier and stored in m_iClosestFungus)
//-----------------------------------------------------------------------
int CNeuralGnat::CheckForFungus(vector<SVector2D> &fungi, double *pDistanceToNeighbour)
{
 if(m_iFrozen)
 {
    return -1;
 }

 SVector2D vDistToObject = fungi[m_iClosestFungus] - m_vWorldPosition;

 if(Vec2DLength(vDistToObject) < 5) // m_dScale)
 {
#ifdef SINGLE_INPUT
    // With only a single input we do not care how close a gnat is 
    // to his neighbour because he has no controll over that.
    *pDistanceToNeighbour = 100.0;
#else
    *pDistanceToNeighbour = m_dDistanceToGnat;
#endif
    return m_iClosestFungus;
 }

 return -1;
}

